/*******************************************************************************


  BATH_CONTROL.C

  DIMMER TEMPORIZADO - PARA CONTROLE DE TEMPERATURA COM PIC 12F675 (60hz)

  AUTOR: CLAUDIO LRIOS

  DATA:  24/04/2015


  OBJETIVO: CONTROLAR TEMPERATURA DO BANHO POR MEIO DE UM POTENCIMETRO DE REGU-
  LAGEM E OPCIONALMENTE, TEMPORIZAR O PERIODO DE BANHO, DESLIGANDO APS ESTE TEM
  PO.

  PROGRAMAR TEMPO DE BANHO EM 6 OPES:
    GIRE O POTENCIMETRO DE REGULAGEM DE TEMPERATURA NO SENTIDO DO CURSOR PARA
    O GND (POTENCIA MXIMA). A CADA APERTO MOMENTANEO DO BOTO 'PROG' SER ESCOLHIDO OUTRA OPO
    DE FORMA SEQUENCIAL:
    1 BIP LONGO  = TIMER DESLIGADO, OPERANDO LIVREMENTE SEM BLOQUEIOS POR TEMPO
    1 BIP CURTO  = 3 MINUTOS DE BANHO
    2 BIPS CURTOS= 5 MINUTOS DE BANHO
    3 BIPS CURTOS= 8 MINUTOS DE BANHO
    4 BIPS CURTOS=11 MINUTOS DE BANHO
    5 BIPS CURTOS=15 MINUTOS DE BANHO


 PROGRAMAR INTERVALO DE BLOQUEIO EM 6 OPES:
    GIRE O POTENCIMETRO DE REGULAGEM DE TEMPERATURA NO SENTIDO DO CURSOR PARA
    O VCC (POTENCIA MNIMA). A CADA APERTO MOMENTANEO DO BOTO 'PROG' SER ESCOLHIDO OUTRA OPO
    DE FORMA SEQUENCIAL:
    1 BIP CURTO  = 2 MINUTOS DE INTERVALO
    2 BIPS CURTOS= 3 MINUTOS DE INTERVALO
    3 BIPS CURTOS= 4 MINUTOS DE INTERVALO
    4 BIPS CURTOS= 5 MINUTOS DE INTERVALO
    5 BIPS CURTOS= 8 MINUTOS DE INTERVALO
    6 BIPS CURTOS=10 MINUTOS DE INTERVALO

 OBS. OCORRER BIP DE AVISO COM 1 MINUTO, 30 SEG E 15 SEGUNDOS ANTES DE DESLIGAR
 O CHUVEIRO (CONFORME A OPO ESCOLHIDA NA DEFINIO DO USURIO).

  ESTE ARQUIVO  PARTE INTEGRANTE DO BLOG 'LARIOS.TECNOLOGIA.WS'

  ATENO: CUIDADOS ESPECIAIS DEVEM SER TOMADOS AO MANUSEAR ESTE CIRCUITO
  EM VIRTUDE DE OPERAR COM ALTAS TENSES, COM RISCOS DE CHOQUES E ELETROCUSSO.
  TRATA-SE DE UM CIRCUITO EXPERIMENTAL APENAS, SUJEITO A BUGS AINDA NO OBSERVA-
  DOS NOS TESTES, MAS QUE PODEM OCORRER NO USO PRTICO.

********************************************************************************
                      Definies do usurio:
*******************************************************************************/

//******************************************************************************
//Descomente para ativar:
#define bipa_com_1minuto_antes_desligar//1 bip  c/ 1 min. faltando p/ desligar
#define bipa_com_30seg_antes_desligar//  2 bips c/ 30 seg. faltando p/ desligar
#define bipa_com_15seg_antes_desligar//  3 bips c/ 15 seg. faltando p/ desligar
//******************************************************************************
//                     Fim das definies de usurio
//******************************************************************************


#include <12F675.h> // pic usado  o 12f675
#device adc=10 //conversor em modo 10 bits
#use delay(clock=4000000)// clock interno de 4 mhz
#use fast_io(a)// entrada e sada definidas pelo programador
#zero_ram  //zera variavis ao iniciar
#rom 0x2100 = {1,0}//pr-gravao de valores iniciais na eeprom
#fuses NOWDT,INTRC_IO, NOCPD, NOPROTECT, NOMCLR, NOPUT, NOBROWNOUT//configurao


//bytes
#byte  gpio    = 0x05
#byte  trisio  = 0x85
#byte  timer0  = 0x01

//bits
#bit  flagtmr0 = 0x0b.2  //flag do timer 0
#bit  in_l     = 0x05.4  //sensor de chuveiro ligado (pino 3)
#bit  in_z     = 0x05.2  //sensor de passagem por 'zero' na rede (pino 5)
#bit  tris_g   = 0x85.0  //direo para acionamento do triac (pino 7)
#bit  prog     = 0x05.3  //entrada do boto 'prog'(pino 4)
#bit  bip      = 0x85.5  //como sada para piezo eltrico (pino 2)
#bit  led_bloq = 0x85.5  //como sada para led  'bloqueado' ao vcc (pino 2)

//variavis globais
int8 tempo,limite;// contadores de tempo para ajuste e limite de excurso
int8 seg_60,seg,min,seg_60_i,seg_i,min_i;
int1 chuv_on,bloq,fled;

//constantes
int8 const  freq_rede = 60;
int8 const fator = 75; //75 para 60Hz //62 para 50Hz
int8 n_opc= 5;//nmero de opes para regulagem

//arrays para tempo de banho e tempo bloqueado (em minutos)
int8 const tempo_banho[]={0,3,5,8,11,15};//valores p/ programao do tempo de banho
int8 const tempo_bloq[]={2,3,4,5,8,10};//valores p/ programao do tempo bloqueado


//------------------------------------------------------------------------------
//          ROTINA DE SADA DOS BIPS
//------------------------------------------------------------------------------
#separate
void bip_out(){bip=0; delay_us(130); bip=1; delay_us(130);}


//------------------------------------------------------------------------------
//          ROTINA GERADORA DE BIPS
//------------------------------------------------------------------------------
void bipa(int8 nbip){
  int16 b,f;
  int8 d;
    if(nbip==0) { for(b=0;b<10000;b++){bip_out();} }// temporizador off
    else{ for(d=0;d<nbip;d++){ for(f=0;f<800;f++){bip_out();} delay_ms(400); }}
}
//------------------------------------------------------------------------------
//        ROTINA DE PROGRAMAO P/ TEMPO DE BANHO E INTERVALO BLOQUEADO
//------------------------------------------------------------------------------
void modo_prog(){ // grava na Eeprom a programao desejada

   if(!prog){
      int16 bb;
      int8 a;
      if(prog) return;
      bb= read_adc();//testa se pot est no mx ou min
      bb=bb>>2;
      a=make8(bb,0);
      if(a<128){// acerta tempo do banho
       a= read_eeprom(0);
       if(++a>n_opc)a=0;
       write_eeprom(0,a);
       bipa(a); //bip longo=desligado + 1 a 5 bips
       }
      else{//acerta tempo do intervalo bloqueado
       a= read_eeprom(1);
       if(++a>n_opc)a=0;
       write_eeprom(1,a);
       bipa(a+1);//bipa de 1 a 6 vezes

      }
      while(!prog)delay_ms(50);//aguarda soltar
   }
}
//------------------------------------------------------------------------------
//          ROTINA DE PULSO PARA O TRIAC
//------------------------------------------------------------------------------

void pulsa_triac(){//rotina de pulso na gate do triac
  int8 a;
   if((tempo<limite)&&!bloq&&chuv_on){//'rajada' de 10 pulsos de ton=25us
   for(a=0; a<10;a++){// p/ evitar a 'derrubada' do vcc c/ o consumo do gate
    tris_g=0; //saida em '0' (liga pulso)
    delay_us(25);//pulso de 25 useg
    tris_g=1;//alta impedancia (corta pulso)
    delay_us(25);//toff de 25 useg
   }
  }
}

//------------------------------------------------------------------------------
//          ROTINA DE DELAY PR-PULSO DO TRIAC
//------------------------------------------------------------------------------
void delay(int8 a){//gera o atraso no disparo correspondente a potencia desejada
  int16 b;
  for(b=0; b<a;b++){
   delay_us(20);//delay 20 useg
  }
}
//------------------------------------------------------------------------------
//          ROTINA DE CONTROLE DE DISPARO PARA O TRIAC
//------------------------------------------------------------------------------

void controla_triac(){
   while(in_z) ;//aguarda mudana no ciclo para -
    set_timer0(0);//seta flag de over flow do timer 0
    flagtmr0=0;//zera timer 0
    delay(tempo);//aguarda momento correto
    pulsa_triac(); //  e dispara o triac
    while(!in_z);//aguarda mudana no ciclo para +
    if(in_l) {chuv_on=1;} else {chuv_on=0;}//testa se chuveiro est ligado (+)
    if(flagtmr0){limite=255;} else{limite=timer0; limite+=fator;}//obtem limite
    delay(tempo);//aguarda momento correto
    pulsa_triac(); //  e dispara o triac
}
//------------------------------------------------------------------------------
//       ROTINA DE LEITURA DO POTENCIMETRO DE REGULAGEM DE TEMPERATURA
//------------------------------------------------------------------------------
void le_pot(){
 int16 valor;
 valor= read_adc();
 valor=valor>>2;
 tempo=make8(valor,0);
 if(tempo>limite)tempo=limite;//testa sw (aumenta)

 }
//------------------------------------------------------------------------------
//       ROTINAS DE CONTROLE DA TEMPORIZAO
//------------------------------------------------------------------------------

#separate
void reset_regs(){//reseta registradores de contagem de tempo
     seg_60_i=0;
     seg_i=0;
     min_i=0;
     seg_60=0;
     seg=0;
     min=0;
}


void temporizar(){//rotina de temporizao do chuveiro
  int8 g;

  if(read_eeprom(0)==0){//testa se temporizao foi desligada pelo usurio
    bloq=0;
    reset_regs();
    return;//timer desligado, sai da rotina de temporizao
  }

  if(!bloq&&chuv_on){//se desbloqueado e gua saindo pelo chuveiro (contatos on)

    if(++seg_60==freq_rede) {
     seg_60=0; if(++seg==60) {seg=0; min++;}//incrementa contadores de tempo on

    g=tempo_banho[read_eeprom(0)]; //le opo de tempo de banho na eeprom
   #ifdef bipa_com_1minuto_antes_desligar
   if((min==(g-1))&&(!seg)&&(!seg_60)) bipa(1);//1 bip :resta 1 min.
   #endif
   #ifdef bipa_com_30seg_antes_desligar
   if((min==(g-1))&&(seg==30)&&(!seg_60)) bipa(2);//2 bips: resta 30 seg.
   #endif
   #ifdef bipa_com_15seg_antes_desligar
   if((min==(g-1))&&(seg==45)&&(!seg_60)) bipa(3);//3 bips:resta 15 seg.
   #endif
   
   if(min>=g){
     bloq=1;//bloqueia
     reset_regs();//reseta registradores de tempo
     bipa(0);//apito longo e desliga chuveiro
     fled=1; //aciona led indicador de bloqueio
   }
  }
 }

  else{ //bloqueado ou chuveiro desligado pelo registro de agua
     if(++seg_60_i==freq_rede) {
     seg_60_i=0; if(++seg_i==60) {seg_i=0; min_i++;}
     g=tempo_bloq[read_eeprom(1)];//le tempo de bloqueio
     if(min_i>=g){
      bloq=0;
      reset_regs();
      if(chuv_on)bipa(0);//bip longo somente se chuveiro ligado pelo registro
      fled=0;//reseta flag do led de bloqueio

    }
   }
  }//else
}//void timer()


void pisca_led_bloq(){
static int8 vez;

 if(fled){
   if(vez++>20){
     vez=0;
     led_bloq=0;
     delay_ms(3);
     }//
  }
  led_bloq=1;//desliga led de bloqueio

}
/*******************************************************************************

                   ROTINA PRINCIPAL (INICIALIZAO)

*******************************************************************************/

void main() {

   setup_adc_ports(sAN1|VSS_VDD);//seleciona gp1 (pino 6) como entrada A/D
   setup_adc(ADC_CLOCK_DIV_8);//ajusta frequncia de converso
   set_adc_channel( 1 );//canal 1 em gp1 pino 6 (le potencimetro de regulagem)
   setup_counters(RTCC_INTERNAL,RTCC_DIV_64); //timer 0 para /64
   setup_timer_1(T1_DISABLED);//desliga timer 1
   setup_comparator(NC_NC_NC_NC );//desliga comparador
   setup_vref(FALSE);//desliga v ref
   gpio=0;//zera pinos de sada
   trisio=0b111111;//sentido dos pinos como entradas
   limite=255;//valor provisrio inicial
   tempo=limite;// inicia com lampada desligada
   delay_ms(1000);//permite a carga do capacitor da fonte
   reset_regs();//reseta registradores de tempo do temporizador
   bloq=0;//desbloqueio inicial ao alimentar o aparelho
   fled=0;//reseta flag
/*******************************************************************************

                  LAO PRINCIPAL DE REPETIO
                      
*******************************************************************************/
   for(;;){
    modo_prog();//verifica se foi acionado o modo de programao
    temporizar();//rotina de temporizao
    le_pot();//obtem valor do potencimetro de ajuste
    controla_triac();//controla a potencia na lampada
    pisca_led_bloq();//testa se deve 'piscar' led bloqueio 
   }//for(;;)

}//main()
